@@ -71,6 +71,7 @@ pub(crate) enum AllocatedOpcode {
     MOD(AllocatedRegister, AllocatedRegister, AllocatedRegister),
     MODI(AllocatedRegister, AllocatedRegister, VirtualImmediate12),
     MOVE(AllocatedRegister, AllocatedRegister),
+    MOVI(AllocatedRegister, VirtualImmediate18),
     MUL(AllocatedRegister, AllocatedRegister, AllocatedRegister),
     MULI(AllocatedRegister, AllocatedRegister, VirtualImmediate12),
     NOT(AllocatedRegister, AllocatedRegister),
@@ -99,13 +100,13 @@ pub(crate) enum AllocatedOpcode {
     MCL(AllocatedRegister, AllocatedRegister),
     MCLI(AllocatedRegister, VirtualImmediate18),
     MCP(AllocatedRegister, AllocatedRegister, AllocatedRegister),
+    MCPI(AllocatedRegister, AllocatedRegister, VirtualImmediate12),
     MEQ(
         AllocatedRegister,
         AllocatedRegister,
         AllocatedRegister,
         AllocatedRegister,
     ),
-    MCPI(AllocatedRegister, AllocatedRegister, VirtualImmediate12),
     SB(AllocatedRegister, AllocatedRegister, VirtualImmediate12),
     SW(AllocatedRegister, AllocatedRegister, VirtualImmediate12),
     BAL(AllocatedRegister, AllocatedRegister, AllocatedRegister),
@@ -134,6 +135,12 @@ pub(crate) enum AllocatedOpcode {
         AllocatedRegister,
         AllocatedRegister,
     ),
+    LOGD(
+        AllocatedRegister,
+        AllocatedRegister,
+        AllocatedRegister,
+        AllocatedRegister,
+    ),
     MINT(AllocatedRegister),
     RVRT(AllocatedRegister),
     SLDC(AllocatedRegister, AllocatedRegister, AllocatedRegister),
@@ -151,7 +158,12 @@ pub(crate) enum AllocatedOpcode {
     ECR(AllocatedRegister, AllocatedRegister, AllocatedRegister),
     K256(AllocatedRegister, AllocatedRegister, AllocatedRegister),
     S256(AllocatedRegister, AllocatedRegister, AllocatedRegister),
+    XIL(AllocatedRegister, AllocatedRegister),
+    XIS(AllocatedRegister, AllocatedRegister),
+    XOL(AllocatedRegister, AllocatedRegister),
     XOS(AllocatedRegister, AllocatedRegister),
+    XWL(AllocatedRegister, AllocatedRegister),
+    XWS(AllocatedRegister, AllocatedRegister),
     NOOP,
     FLAG(AllocatedRegister),
     GM(AllocatedRegister, VirtualImmediate18),
@@ -189,6 +201,7 @@ impl fmt::Display for AllocatedOp {
             MOD(a, b, c)    => format!("mod  {} {} {}", a, b, c),
             MODI(a, b, c)   => format!("modi {} {} {}", a, b, c),
             MOVE(a, b)      => format!("move {} {}", a, b),
+            MOVI(a, b)      => format!("movi {} {}", a, b),
             MUL(a, b, c)    => format!("mul  {} {} {}", a, b, c),
             MULI(a, b, c)   => format!("muli {} {} {}", a, b, c),
             NOT(a, b)       => format!("not  {} {}", a, b),
@@ -217,8 +230,8 @@ impl fmt::Display for AllocatedOp {
             MCL(a, b)       => format!("mcl  {} {}", a, b),
             MCLI(a, b)      => format!("mcli {} {}", a, b),
             MCP(a, b, c)    => format!("mcp  {} {} {}", a, b, c),
-            MEQ(a, b, c, d) => format!("meq  {} {} {} {}", a, b, c, d),
             MCPI(a, b, c)   => format!("mcpi {} {} {}", a, b, c),
+            MEQ(a, b, c, d) => format!("meq  {} {} {} {}", a, b, c, d),
             SB(a, b, c)     => format!("sb   {} {} {}", a, b, c),
             SW(a, b, c)     => format!("sw   {} {} {}", a, b, c),
             BAL(a, b, c)    => format!("bal  {} {} {}", a, b, c),
@@ -232,6 +245,7 @@ impl fmt::Display for AllocatedOp {
             CB(a)           => format!("cb   {}", a),
             LDC(a, b, c)    => format!("ldc  {} {} {}", a, b, c),
             LOG(a, b, c, d) => format!("log  {} {} {} {}", a, b, c, d),
+            LOGD(a, b, c, d)=> format!("logd {} {} {} {}", a, b, c, d),
             MINT(a)         => format!("mint {}", a),
             RVRT(a)         => format!("rvrt {}", a),
             SLDC(a, b, c)   => format!("sldc {} {} {}", a, b, c),
@@ -244,13 +258,18 @@ impl fmt::Display for AllocatedOp {
             ECR(a, b, c)    => format!("ecr  {} {} {}", a, b, c),
             K256(a, b, c)   => format!("k256 {} {} {}", a, b, c),
             S256(a, b, c)   => format!("s256 {} {} {}", a, b, c),
+            XIL(a, b)       => format!("xil {} {}", a, b),
+            XIS(a, b)       => format!("xis {} {}", a, b),
+            XOL(a, b)       => format!("xol {} {}", a, b),
             XOS(a, b)       => format!("xos {} {}", a, b),
+            XWL(a, b)       => format!("xwl {} {}", a, b),
+            XWS(a, b)       => format!("xws {} {}", a, b),
             NOOP            => "noop".to_string(),
             FLAG(a)         => format!("flag {}", a),
             GM(a, b)         => format!("gm {} {}", a, b),
             Undefined       => "undefined op".into(),
             DataSectionOffsetPlaceholder => "DATA_SECTION_OFFSET[0..32]\nDATA_SECTION_OFFSET[32..64]".into(),
-            DataSectionRegisterLoadPlaceholder => "lw   $ds $is 1".into()
+            DataSectionRegisterLoadPlaceholder => "lw   $ds $is 1".into(),
         };
         // we want the comment to always be COMMENT_START_COLUMN characters offset to the right
         // to not interfere with the ASM but to be aligned
@@ -293,6 +312,7 @@ impl AllocatedOp {
             MOD (a, b, c)   => VmOp::MOD (a.to_register_id(), b.to_register_id(), c.to_register_id()),
             MODI(a, b, c)   => VmOp::MODI(a.to_register_id(), b.to_register_id(), c.value),
             MOVE(a, b)      => VmOp::MOVE(a.to_register_id(), b.to_register_id()),
+            MOVI(a, b)      => VmOp::MOVI(a.to_register_id(), b.value),
             MUL (a, b, c)   => VmOp::MUL (a.to_register_id(), b.to_register_id(), c.to_register_id()),
             MULI(a, b, c)   => VmOp::MULI(a.to_register_id(), b.to_register_id(), c.value),
             NOT (a, b)      => VmOp::NOT (a.to_register_id(), b.to_register_id()),
@@ -321,8 +341,8 @@ impl AllocatedOp {
             MCL (a, b)      => VmOp::MCL (a.to_register_id(), b.to_register_id()),
             MCLI(a, b)      => VmOp::MCLI(a.to_register_id(), b.value),
             MCP (a, b, c)   => VmOp::MCP (a.to_register_id(), b.to_register_id(), c.to_register_id()),
-            MEQ (a, b, c, d)=> VmOp::MEQ (a.to_register_id(), b.to_register_id(), c.to_register_id(), d.to_register_id()),
             MCPI(a, b, c)   => VmOp::MCPI(a.to_register_id(), b.to_register_id(), c.value),
+            MEQ (a, b, c, d)=> VmOp::MEQ (a.to_register_id(), b.to_register_id(), c.to_register_id(), d.to_register_id()),
             SB  (a, b, c)   => VmOp::SB  (a.to_register_id(), b.to_register_id(), c.value),
             SW  (a, b, c)   => VmOp::SW  (a.to_register_id(), b.to_register_id(), c.value),
             BAL  (a, b, c)  => VmOp::BAL (a.to_register_id(), b.to_register_id(), c.to_register_id()),
@@ -336,6 +356,7 @@ impl AllocatedOp {
             CB  (a)         => VmOp::CB  (a.to_register_id()),
             LDC (a, b, c)   => VmOp::LDC (a.to_register_id(), b.to_register_id(), c.to_register_id()),
             LOG (a, b, c, d)=> VmOp::LOG (a.to_register_id(), b.to_register_id(), c.to_register_id(), d.to_register_id()),
+            LOGD(a, b, c, d)=> VmOp::LOGD(a.to_register_id(), b.to_register_id(), c.to_register_id(), d.to_register_id()),
             MINT(a)         => VmOp::MINT(a.to_register_id()),
             RVRT(a)         => VmOp::RVRT(a.to_register_id()),
             SLDC(a, b, c)   => VmOp::SLDC(a.to_register_id(), b.to_register_id(), c.to_register_id()),
@@ -348,7 +369,12 @@ impl AllocatedOp {
             ECR (a, b, c)   => VmOp::ECR (a.to_register_id(), b.to_register_id(), c.to_register_id()),
             K256(a, b, c)   => VmOp::K256(a.to_register_id(), b.to_register_id(), c.to_register_id()),
             S256(a, b, c)   => VmOp::S256(a.to_register_id(), b.to_register_id(), c.to_register_id()),
-            XOS(a, b)       => VmOp::XOS(a.to_register_id(), b.to_register_id()),
+            XIL (a, b)      => VmOp::XIL(a.to_register_id(), b.to_register_id()),
+            XIS (a, b)      => VmOp::XIS(a.to_register_id(), b.to_register_id()),
+            XOL (a, b)      => VmOp::XOL(a.to_register_id(), b.to_register_id()),
+            XOS (a, b)      => VmOp::XOS(a.to_register_id(), b.to_register_id()),
+            XWL (a, b)      => VmOp::XWL(a.to_register_id(), b.to_register_id()),
+            XWS (a, b)      => VmOp::XWS(a.to_register_id(), b.to_register_id()),
             NOOP            => VmOp::NOOP,
             FLAG(a)         => VmOp::FLAG(a.to_register_id()),
             GM(a, b)         => VmOp::GM(a.to_register_id(), b.value),
