@@ -37,7 +37,7 @@ macro_rules! check_std_result {
 }
 
 macro_rules! assert_or_warn {
-    ($bool_expr: expr, $warnings: ident, $span: expr, $warning: expr $(,)?) => {
+    ($bool_expr: expr, $warnings: ident, $span: expr, $warning: expr $(,)?) => {{
         if !$bool_expr {
             use crate::error::CompileWarning;
             $warnings.push(CompileWarning {
@@ -45,7 +45,7 @@ macro_rules! assert_or_warn {
                 span: $span,
             });
         }
-    };
+    }};
 }
 
 /// Denotes a non-recoverable state
@@ -57,15 +57,6 @@ pub(crate) fn err<T>(warnings: Vec<CompileWarning>, errors: Vec<CompileError>) -
     }
 }
 
-pub(crate) fn infallible<T>(res: CompileResult<T>) -> T {
-    match res.value {
-        Some(x) if res.errors.is_empty() && res.warnings.is_empty() => x,
-        _ => {
-            panic!("Internal compiler error: called `infallible` on a fallible compile result. Please report this bug: github.com/FuelLabs/Sway/issues")
-        }
-    }
-}
-
 /// Denotes a recovered or non-error state
 pub(crate) fn ok<T>(
     value: T,
@@ -227,6 +218,9 @@ pub enum Warning {
     NonClassCaseStructName {
         struct_name: Ident,
     },
+    NonClassCaseTypeParameter {
+        name: Ident,
+    },
     NonClassCaseTraitName {
         name: Ident,
     },
@@ -274,6 +268,7 @@ pub enum Warning {
     ShadowingReservedRegister {
         reg_name: Ident,
     },
+    DeadStorageDeclaration,
     MatchExpressionUnreachableArm,
 }
 
@@ -290,6 +285,14 @@ impl fmt::Display for Warning {
                 to_upper_camel_case(struct_name.as_str())
             )
             }
+            NonClassCaseTypeParameter { name } => {
+                write!(f,
+                "Type parameter \"{}\" is not idiomatic. TypeParameters should have a ClassCase name, like \
+                 \"{}\".",
+                name,
+                to_upper_camel_case(name.as_str())
+            )
+            }
             NonClassCaseTraitName { name } => {
                 write!(f,
                 "Trait name \"{}\" is not idiomatic. Traits should have a ClassCase name, like \
@@ -380,6 +383,8 @@ impl fmt::Display for Warning {
                 "This register declaration shadows the reserved register, \"{}\".",
                 reg_name
             ),
+            DeadStorageDeclaration => write!(f, "This storage declaration is never accessed and can be removed."
+            ),
             MatchExpressionUnreachableArm => write!(f, "This match arm is unreachable."),
         }
     }
@@ -537,7 +542,7 @@ pub enum CompileError {
         given: String,
         expected: String,
     },
-    #[error("\"{name}\" is not a trait, so it cannot be \"impl'd\". ")]
+    #[error("\"{name}\" is not a trait, so it cannot be \"impl'd\".")]
     NotATrait { span: Span, name: Ident },
     #[error("Trait \"{name}\" cannot be found in the current scope.")]
     UnknownTrait { span: Span, name: Ident },
@@ -558,6 +563,10 @@ pub enum CompileError {
         expected: usize,
         span: Span,
     },
+    #[error("\"{name}\" does not take type arguments.")]
+    DoesNotTakeTypeArguments { name: Ident, span: Span },
+    #[error("\"{name}\" needs type arguments.")]
+    NeedsTypeArguments { name: Ident, span: Span },
     #[error(
         "Struct with name \"{name}\" could not be found in this scope. Perhaps you need to import \
          it?"
@@ -604,6 +613,8 @@ pub enum CompileError {
         method_name: String,
         type_name: String,
     },
+    #[error("Duplicate definitions with name \"{method_name}\".")]
+    DuplicateMethodDefinitions { method_name: String, span: Span },
     #[error("Module \"{name}\" could not be found.")]
     ModuleNotFound { span: Span, name: String },
     #[error("\"{name}\" is a {actually}, not a struct. Fields can only be accessed on structs.")]
@@ -891,6 +902,14 @@ pub enum CompileError {
     UnrecognizedContractParam { param_name: String, span: Span },
     #[error("Attempting to specify a contract method parameter for a non-contract function call")]
     CallParamForNonContractCallMethod { span: Span },
+    #[error("Storage field {name} does not exist")]
+    StorageFieldDoesNotExist { name: String, span: Span },
+    #[error("No storage has been declared")]
+    NoDeclaredStorage { span: Span },
+    #[error("Multiple storage declarations were found")]
+    MultipleStorageDeclarations { span: Span },
+    #[error("Expected identifier, found keyword \"{name}\" ")]
+    InvalidVariableName { name: String, span: Span },
 }
 
 impl std::convert::From<TypeError> for CompileError {
@@ -1016,6 +1035,8 @@ impl CompileError {
             FunctionNotAPartOfInterfaceSurface { span, .. } => span,
             MissingInterfaceSurfaceMethods { span, .. } => span,
             IncorrectNumberOfTypeArguments { span, .. } => span,
+            DoesNotTakeTypeArguments { span, .. } => span,
+            NeedsTypeArguments { span, .. } => span,
             StructNotFound { span, .. } => span,
             DeclaredNonStructAsStruct { span, .. } => span,
             AccessedFieldOfNonStruct { span, .. } => span,
@@ -1023,6 +1044,7 @@ impl CompileError {
             StructMissingField { span, .. } => span,
             StructDoesNotHaveField { span, .. } => span,
             MethodNotFound { span, .. } => span,
+            DuplicateMethodDefinitions { span, .. } => span,
             ModuleNotFound { span, .. } => span,
             NotATuple { span, .. } => span,
             NotAStruct { span, .. } => span,
@@ -1099,6 +1121,10 @@ impl CompileError {
             ContractCallParamRepeated { span, .. } => span,
             UnrecognizedContractParam { span, .. } => span,
             CallParamForNonContractCallMethod { span, .. } => span,
+            StorageFieldDoesNotExist { span, .. } => span,
+            NoDeclaredStorage { span, .. } => span,
+            MultipleStorageDeclarations { span, .. } => span,
+            InvalidVariableName { span, .. } => span,
         }
     }
 
