@@ -16,7 +16,6 @@ pub struct ImplTrait {
     pub functions: Vec<FunctionDeclaration>,
     // the span of the whole impl trait and block
     pub(crate) block_span: Span,
-    pub(crate) type_arguments_span: Span,
 }
 
 /// An impl of methods without a trait
@@ -24,15 +23,11 @@ pub struct ImplTrait {
 #[derive(Debug, Clone)]
 pub struct ImplSelf {
     pub(crate) type_implementing_for: TypeInfo,
-    pub(crate) type_arguments: Vec<TypeParameter>,
+    pub(crate) type_implementing_for_span: Span,
+    pub(crate) type_parameters: Vec<TypeParameter>,
     pub functions: Vec<FunctionDeclaration>,
     // the span of the whole impl trait and block
     pub(crate) block_span: Span,
-    #[allow(dead_code)]
-    // these spans may be used for errors in the future, although it is not right now.
-    pub(crate) type_arguments_span: Span,
-    #[allow(dead_code)]
-    pub(crate) type_name_span: Span,
 }
 
 impl ImplTrait {
@@ -40,16 +35,27 @@ impl ImplTrait {
         pair: Pair<Rule>,
         config: Option<&BuildConfig>,
     ) -> CompileResult<Self> {
-        let path = config.map(|c| c.path());
         let mut warnings = Vec::new();
         let mut errors = Vec::new();
+
+        let path = config.map(|c| c.path());
         let block_span = Span {
             span: pair.as_span(),
             path: path.clone(),
         };
-        let mut iter = pair.into_inner();
+
+        let mut iter = pair.into_inner().peekable();
+
         let impl_keyword = iter.next().unwrap();
         assert_eq!(impl_keyword.as_str(), "impl");
+
+        // see if there are any generic type params
+        let type_params_pair = match iter.peek() {
+            Some(pair) if pair.as_rule() == Rule::type_params => Some(iter.next().unwrap()),
+            _ => None,
+        };
+
+        // get the trait name
         let trait_name = iter.next().unwrap();
         let trait_name = check!(
             CallPath::parse_from_pair(trait_name, config),
@@ -57,49 +63,40 @@ impl ImplTrait {
             warnings,
             errors
         );
-        let mut iter = iter.peekable();
-        let type_params_pair = if iter.peek().unwrap().as_rule() == Rule::type_params {
-            iter.next()
-        } else {
-            None
-        };
 
-        let type_implementing_for_pair = iter.next().expect("guaranteed by grammar");
+        // construct the type that we are implementing for
+        let type_name = iter.next().unwrap();
         let type_implementing_for_span = Span {
-            span: type_implementing_for_pair.as_span(),
-            path: path.clone(),
+            span: type_name.as_span(),
+            path,
         };
         let type_implementing_for = check!(
-            TypeInfo::parse_from_pair(type_implementing_for_pair, config),
+            TypeInfo::parse_from_pair(type_name, config),
             return err(warnings, errors),
             warnings,
             errors
         );
 
+        // see if there are any trait bounds
         let where_clause_pair = match iter.peek() {
-            Some(r) => match r.as_rule() {
-                Rule::trait_bounds => iter.next(),
-                _ => None,
-            },
-            None => None,
+            Some(pair) if pair.as_rule() == Rule::trait_bounds => Some(iter.next().unwrap()),
+            _ => None,
         };
 
-        let type_arguments_span = match type_params_pair {
-            Some(ref x) => Span {
-                span: x.as_span(),
-                path,
-            },
-            None => trait_name.span(),
-        };
-        let type_arguments = TypeParameter::parse_from_type_params_and_where_clause(
-            type_params_pair,
-            where_clause_pair,
-            config,
-        )
-        .unwrap_or_else(&mut warnings, &mut errors, Vec::new);
+        // construct the type arguments
+        let type_arguments = check!(
+            TypeParameter::parse_from_type_params_and_where_clause(
+                type_params_pair,
+                where_clause_pair,
+                config
+            ),
+            vec!(),
+            warnings,
+            errors
+        );
 
+        // collect the methods in the impl
         let mut fn_decls_buf = vec![];
-
         for pair in iter {
             fn_decls_buf.push(check!(
                 FunctionDeclaration::parse_from_pair(pair, config),
@@ -113,7 +110,6 @@ impl ImplTrait {
             ImplTrait {
                 trait_name,
                 type_arguments,
-                type_arguments_span,
                 type_implementing_for,
                 type_implementing_for_span,
                 functions: fn_decls_buf,
@@ -130,55 +126,59 @@ impl ImplSelf {
         pair: Pair<Rule>,
         config: Option<&BuildConfig>,
     ) -> CompileResult<Self> {
-        let path = config.map(|c| c.path());
         let mut warnings = Vec::new();
         let mut errors = Vec::new();
+
+        let path = config.map(|c| c.path());
         let block_span = Span {
             span: pair.as_span(),
             path: path.clone(),
         };
-        let mut iter = pair.into_inner();
+
+        let mut iter = pair.into_inner().peekable();
+
         let impl_keyword = iter.next().unwrap();
         assert_eq!(impl_keyword.as_str(), "impl");
-        let mut iter = iter.peekable();
-        let type_params_pair = if iter.peek().unwrap().as_rule() == Rule::type_params {
-            iter.next()
-        } else {
-            None
-        };
-        let type_pair = iter.next().unwrap();
-        let type_name_span = Span {
-            span: type_pair.as_span(),
-            path: path.clone(),
+
+        // see if there are any generic type params
+        let type_params_pair = match iter.peek() {
+            Some(pair) if pair.as_rule() == Rule::type_params => Some(iter.next().unwrap()),
+            _ => None,
         };
 
+        // construct the type that we are implementing for
+        let type_name = iter.next().unwrap();
+        let type_implementing_for_span = Span {
+            span: type_name.as_span(),
+            path,
+        };
         let type_implementing_for = check!(
-            TypeInfo::parse_from_pair(type_pair, config),
+            TypeInfo::parse_from_pair(type_name, config),
             return err(warnings, errors),
             warnings,
             errors
         );
 
+        // see if there are any trait bounds
         let where_clause_pair = match iter.peek() {
-            Some(pair) if pair.as_rule() == Rule::trait_bounds => iter.next(),
+            Some(pair) if pair.as_rule() == Rule::trait_bounds => Some(iter.next().unwrap()),
             _ => None,
         };
-        let type_arguments_span = match type_params_pair {
-            Some(ref x) => Span {
-                span: x.as_span(),
-                path,
-            },
-            None => type_name_span.clone(),
-        };
-        let type_arguments = TypeParameter::parse_from_type_params_and_where_clause(
-            type_params_pair,
-            where_clause_pair,
-            config,
-        )
-        .unwrap_or_else(&mut warnings, &mut errors, Vec::new);
 
+        // construct the type arguments
+        let type_arguments = check!(
+            TypeParameter::parse_from_type_params_and_where_clause(
+                type_params_pair,
+                where_clause_pair,
+                config
+            ),
+            vec!(),
+            warnings,
+            errors
+        );
+
+        // collect the methods in the impl
         let mut fn_decls_buf = vec![];
-
         for pair in iter {
             fn_decls_buf.push(check!(
                 FunctionDeclaration::parse_from_pair(pair, config),
@@ -190,11 +190,10 @@ impl ImplSelf {
 
         ok(
             ImplSelf {
-                type_arguments,
-                type_arguments_span,
                 type_implementing_for,
+                type_implementing_for_span,
+                type_parameters: type_arguments,
                 functions: fn_decls_buf,
-                type_name_span,
                 block_span,
             },
             warnings,
