@@ -10,12 +10,13 @@ use pest::Span;
 
 use std::{
     convert::TryInto,
+    hash::{Hash, Hasher},
     num::{IntErrorKind, ParseIntError},
     path::PathBuf,
     sync::Arc,
 };
 
-#[derive(Debug, Clone, Eq, PartialEq)]
+#[derive(Debug, Clone, Eq)]
 pub enum Literal {
     U8(u8),
     U16(u16),
@@ -28,6 +29,73 @@ pub enum Literal {
     B256([u8; 32]),
 }
 
+// NOTE: Hash and PartialEq must uphold the invariant:
+// k1 == k2 -> hash(k1) == hash(k2)
+// https://doc.rust-lang.org/std/collections/struct.HashMap.html
+impl Hash for Literal {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        use Literal::*;
+        match self {
+            U8(x) => {
+                state.write_u8(1);
+                x.hash(state);
+            }
+            U16(x) => {
+                state.write_u8(2);
+                x.hash(state);
+            }
+            U32(x) => {
+                state.write_u8(3);
+                x.hash(state);
+            }
+            U64(x) => {
+                state.write_u8(4);
+                x.hash(state);
+            }
+            Numeric(x) => {
+                state.write_u8(5);
+                x.hash(state);
+            }
+            String(inner) => {
+                state.write_u8(6);
+                inner.as_str().hash(state);
+            }
+            Boolean(x) => {
+                state.write_u8(7);
+                x.hash(state);
+            }
+            Byte(x) => {
+                state.write_u8(8);
+                x.hash(state);
+            }
+            B256(x) => {
+                state.write_u8(9);
+                x.hash(state);
+            }
+        }
+    }
+}
+
+// NOTE: Hash and PartialEq must uphold the invariant:
+// k1 == k2 -> hash(k1) == hash(k2)
+// https://doc.rust-lang.org/std/collections/struct.HashMap.html
+impl PartialEq for Literal {
+    fn eq(&self, other: &Self) -> bool {
+        match (self, other) {
+            (Self::U8(l0), Self::U8(r0)) => l0 == r0,
+            (Self::U16(l0), Self::U16(r0)) => l0 == r0,
+            (Self::U32(l0), Self::U32(r0)) => l0 == r0,
+            (Self::U64(l0), Self::U64(r0)) => l0 == r0,
+            (Self::String(l0), Self::String(r0)) => *l0.as_str() == *r0.as_str(),
+            (Self::Numeric(l0), Self::Numeric(r0)) => l0 == r0,
+            (Self::Boolean(l0), Self::Boolean(r0)) => l0 == r0,
+            (Self::Byte(l0), Self::Byte(r0)) => l0 == r0,
+            (Self::B256(l0), Self::B256(r0)) => l0 == r0,
+            _ => false,
+        }
+    }
+}
+
 impl Literal {
     #[allow(dead_code)]
     pub(crate) fn as_type(&self) -> ResolvedType {
