@@ -1,6 +1,6 @@
 use super::{TypedCodeBlock, TypedExpression};
 
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq)]
 pub(crate) struct TypedWhileLoop {
     pub(crate) condition: TypedExpression,
     pub(crate) body: TypedCodeBlock,
