@@ -369,10 +369,14 @@ impl Dependencies {
                 })
             }
             Expression::SubfieldExpression { prefix, .. } => self.gather_from_expr(prefix),
-            Expression::DelineatedPath { call_path, .. } => {
+            Expression::DelineatedPath {
+                call_path, args, ..
+            } => {
                 // It's either a module path which we can ignore, or an enum variant path, in which
-                // case we're interested in the enum name, ignoring the variant name.
+                // case we're interested in the enum name and initialiser args, ignoring the
+                // variant name.
                 self.gather_from_call_path(call_path, true, false)
+                    .gather_from_iter(args.iter(), |deps, arg| deps.gather_from_expr(arg))
             }
             Expression::MethodApplication { arguments, .. } => {
                 self.gather_from_iter(arguments.iter(), |deps, arg| deps.gather_from_expr(arg))
